/**
  ******************************************************************************
  * @ļ    stm32f10x_flash.c
  * @    MCD Ӧó
  * @汾    V3.1.0
  * @    06/19/2009
  * @    ļ ṩ FLASH ̼.
  * @    ANSON/sweet1985 Email:airanson110@126.com
  * @    23/07/2009
  ******************************************************************************
  * @
  *
  * ̼ṩͻΪƲƷдĲοĿʹͻԼʱ䡣
  * ͻʹñ̼ڿƷϲĽⷨ뵼幫˾еκֱӵĺͼ
  * ΣҲеκ𺦶⳥
  *
  * <h2><center>&; Ȩ 2009 ⷨ뵼幫˾</center></h2>
  * 汾ѧϰӢԭгӦӢԭΪ׼
  */ 

/*  ------------------------------------------------------------------*/
#include "stm32f10x_flash.h"

/*  --------------------------------------------------------------*/
/* ö --------------------------------------------------------------*/ 

/* Flash Access Control Register bits */
#define ACR_LATENCY_Mask         ((uint32_t)0x00000038)
#define ACR_HLFCYA_Mask          ((uint32_t)0xFFFFFFF7)
#define ACR_PRFTBE_Mask          ((uint32_t)0xFFFFFFEF)

/* Flash Access Control Register bits */
#define ACR_PRFTBS_Mask          ((uint32_t)0x00000020) 

/* Flash Control Register bits */
#define CR_PG_Set                ((uint32_t)0x00000001)
#define CR_PG_Reset              ((uint32_t)0x00001FFE) 
#define CR_PER_Set               ((uint32_t)0x00000002)
#define CR_PER_Reset             ((uint32_t)0x00001FFD)
#define CR_MER_Set               ((uint32_t)0x00000004)
#define CR_MER_Reset             ((uint32_t)0x00001FFB)
#define CR_OPTPG_Set             ((uint32_t)0x00000010)
#define CR_OPTPG_Reset           ((uint32_t)0x00001FEF)
#define CR_OPTER_Set             ((uint32_t)0x00000020)
#define CR_OPTER_Reset           ((uint32_t)0x00001FDF)
#define CR_STRT_Set              ((uint32_t)0x00000040)
#define CR_LOCK_Set              ((uint32_t)0x00000080)

/* FLASH Mask */
#define RDPRT_Mask               ((uint32_t)0x00000002)
#define WRP0_Mask                ((uint32_t)0x000000FF)
#define WRP1_Mask                ((uint32_t)0x0000FF00)
#define WRP2_Mask                ((uint32_t)0x00FF0000)
#define WRP3_Mask                ((uint32_t)0xFF000000)

/* FLASH Keys */
#define RDP_Key                  ((uint16_t)0x00A5)
#define FLASH_KEY1               ((uint32_t)0x45670123)
#define FLASH_KEY2               ((uint32_t)0xCDEF89AB)

/* Delay definition */   
#define EraseTimeout             ((uint32_t)0x00000FFF)
#define ProgramTimeout           ((uint32_t)0x0000000F)

/* ú -------------------------------------------------------------*/
/* ñ -----------------------------------------------------------*/
/* úԭ -------------------------------------------------------*/
/* ú -----------------------------------------------------------*/
static void delay(void);
/**
  * @  ôʱֵ.
  * @  FLASH_Latency: ָ FLASH ʱֵ.
  *          ֵ֮һ:
  *          FLASH_Latency_0: FLASH 0ʱ
  *          FLASH_Latency_1: FLASH 1ʱ
  *          FLASH_Latency_2: FLASH 2ʱ
  * @  û
  */
void FLASH_SetLatency(uint32_t FLASH_Latency)
{
  uint32_t tmpreg = 0;
  
  /*  */
  assert_param(IS_FLASH_LATENCY(FLASH_Latency));
  
  /*  ACR Ĵ */
  tmpreg = FLASH->ACR;  
  
  /* ʱֵ */
  tmpreg &= ACR_LATENCY_Mask;
  tmpreg |= FLASH_Latency;
  
  /* д ACR Ĵ */
  FLASH->ACR = tmpreg;
}

/**
  * @  ʹܻʧ FLASH ڷ.
  * @  FLASH_HalfCycleAccess: ָ FLASH ڷģʽ.
  *          ֵ֮һ:
  *          FLASH_HalfCycleAccess_Enable:  ڷʹ
  *          FLASH_HalfCycleAccess_Disable: ڷʧ
  * @  û
  */
void FLASH_HalfCycleAccessCmd(uint32_t FLASH_HalfCycleAccess)
{
  /*  */
  assert_param(IS_FLASH_HALFCYCLEACCESS_STATE(FLASH_HalfCycleAccess));
  
  /* ʹܻʧܰڷ */
  FLASH->ACR &= ACR_HLFCYA_Mask;
  FLASH->ACR |= FLASH_HalfCycleAccess;
}

/**
  * @  ʹܻʧԤȡ.
  * @  FLASH_PrefetchBuffer: ָԤȡ״̬.
  *          ֵ֮һ:
  *          FLASH_PrefetchBuffer_Enable:  ʹԤȡָ
  *          FLASH_PrefetchBuffer_Disable: ʧԤȡָ
  * @  û
  */
void FLASH_PrefetchBufferCmd(uint32_t FLASH_PrefetchBuffer)
{
  /*  */
  assert_param(IS_FLASH_PREFETCHBUFFER_STATE(FLASH_PrefetchBuffer));
  
  /* ʹܻʧԤȡ */
  FLASH->ACR &= ACR_PRFTBE_Mask;
  FLASH->ACR |= FLASH_PrefetchBuffer;
}

/**
  * @   FLASH д.
  * @  û
  * @  û
  */
void FLASH_Unlock(void)
{
  /* Ȩ FPEC ȡ */
  FLASH->KEYR = FLASH_KEY1;
  FLASH->KEYR = FLASH_KEY2;
}

/**
  * @   FLASH д.
  * @  û
  * @  û
  */
void FLASH_Lock(void)
{
  /* λ FPEC  FCR */
  FLASH->CR |= CR_LOCK_Set;
}

/**
  * @  һ FLASH ҳ.
  * @  Page_Address: Ҫҳ.
  * @  FLASH ״̬: ֵ: FLASH_BUSY, FLASH_ERROR_PG,
  *                    FLASH_ERROR_WRP, FLASH_COMPLETE  FLASH_TIMEOUT.
  */
FLASH_Status FLASH_ErasePage(uint32_t Page_Address)
{
  FLASH_Status status = FLASH_COMPLETE;
  /*  */
  assert_param(IS_FLASH_ADDRESS(Page_Address));
  /* ȴһ */
  status = FLASH_WaitForLastOperation(EraseTimeout);
  
  if(status == FLASH_COMPLETE)
  { 
    /* ǰһѾ */
    FLASH->CR|= CR_PER_Set;
    FLASH->AR = Page_Address; 
    FLASH->CR|= CR_STRT_Set;
    
    /* ȴһ */
    status = FLASH_WaitForLastOperation(EraseTimeout);
    if(status != FLASH_TIMEOUT)
    {
      /* ɣʧ PER λ */
      FLASH->CR &= CR_PER_Reset;
    }
  }
  /* ز״̬ */
  return status;
}

/**
  * @  ȫ FLASH ҳ.
  * @  û
  * @  FLASH ״̬: ֵ: FLASH_ERROR_PG, FLASH_ERROR_WRP, 
  *                    FLASH_COMPLETE  FLASH_TIMEOUT.
  */
FLASH_Status FLASH_EraseAllPages(void)
{
  FLASH_Status status = FLASH_COMPLETE;
  /* ȴһ */
  status = FLASH_WaitForLastOperation(EraseTimeout);
  
  if(status == FLASH_COMPLETE)
  {
    /* ǰһѾɣеҳ */
     FLASH->CR |= CR_MER_Set;
     FLASH->CR |= CR_STRT_Set;
    
    /* ȴһ */
    status = FLASH_WaitForLastOperation(EraseTimeout);
    if(status != FLASH_TIMEOUT)
    {
      /* ɣʧ MER λ */
      FLASH->CR &= CR_MER_Reset;
    }
  }	   
  /* ز״̬ */
  return status;
}

/**
  * @   FLASH ѡֽ.
  * @  û
  * @  FLASH Status: ֵ: FLASH_ERROR_PG, FLASH_ERROR_WRP, 
  *                      FLASH_COMPLETE  FLASH_TIMEOUT.
  */
FLASH_Status FLASH_EraseOptionBytes(void)
{
  FLASH_Status status = FLASH_COMPLETE;
  
  /* ȴһ */
  status = FLASH_WaitForLastOperation(EraseTimeout);
  if(status == FLASH_COMPLETE)
  {
    /* ȨСϢ */
    FLASH->OPTKEYR = FLASH_KEY1;
    FLASH->OPTKEYR = FLASH_KEY2;
    
    /* ǰһɣѡֽ */
    FLASH->CR |= CR_OPTER_Set;
    FLASH->CR |= CR_STRT_Set;
    /* ȴһ */
    status = FLASH_WaitForLastOperation(EraseTimeout);
    
    if(status == FLASH_COMPLETE)
    {
      /* ɣʧ OPTER λ */
      FLASH->CR &= CR_OPTER_Reset;
       
      /* ʹֽڱ̲ѡ */
      FLASH->CR |= CR_OPTPG_Set;
      /* ʹܶ */
      OB->RDP= RDP_Key; 
      /* ȴһ */
      status = FLASH_WaitForLastOperation(ProgramTimeout);
 
      if(status != FLASH_TIMEOUT)
      {
        /* ̲ɣʧ OPTPG λ */
        FLASH->CR &= CR_OPTPG_Reset;
      }
    }
    else
    {
      if (status != FLASH_TIMEOUT)
      {
        /* ʧ OPTPG λ */
        FLASH->CR &= CR_OPTPG_Reset;
      }
    }  
  }
  /* ز״̬ */
  return status;
}

/**
  * @  ַָдһ.
  * @  Address: Ҫ̵ĵַ.
  * @  Data: ̵ָ.
  * @  FLASH Status: ֵ: FLASH_ERROR_PG, FLASH_ERROR_WRP, 
  *                      FLASH_COMPLETE  FLASH_TIMEOUT. 
  */
FLASH_Status FLASH_ProgramWord(uint32_t Address, uint32_t Data)
{
  FLASH_Status status = FLASH_COMPLETE;
  __IO uint32_t tmp = 0;

  /*  */
  assert_param(IS_FLASH_ADDRESS(Address));
  /* ȴһ */
  status = FLASH_WaitForLastOperation(ProgramTimeout);
  
  if(status == FLASH_COMPLETE)
  {
    /* ǰһɣµĵһ */
    FLASH->CR |= CR_PG_Set;
  
    *(__IO uint16_t*)Address = (uint16_t)Data;
    /* ȴһ */
    status = FLASH_WaitForLastOperation(ProgramTimeout);
 
    if(status == FLASH_COMPLETE)
    {
      /* ǰһɣµĵڶ */
      tmp = Address + 2;

      *(__IO uint16_t*) tmp = Data >> 16;
    
      /* ȴһ */
      status = FLASH_WaitForLastOperation(ProgramTimeout);
        
      if(status != FLASH_TIMEOUT)
      {
        /* ʧ PG λ */
        FLASH->CR &= CR_PG_Reset;
      }
    }
    else
    {
      if (status != FLASH_TIMEOUT)
      {
        /* ʧ PG λ */
        FLASH->CR &= CR_PG_Reset;
      }
     }
  }
  /* ر״̬ */
  return status;
}

/**
  * @  ضַһ.
  * @  Address: ̵ĵַ.
  * @  Data: ̵ָİ.
  * @  FLASH Status: ֵ: FLASH_ERROR_PG, FLASH_ERROR_WRP, 
  *                      FLASH_COMPLETE  FLASH_TIMEOUT. 
  */
FLASH_Status FLASH_ProgramHalfWord(uint32_t Address, uint16_t Data)
{
  FLASH_Status status = FLASH_COMPLETE;
  /*  */
  assert_param(IS_FLASH_ADDRESS(Address));
  /* ȴһ */
  status = FLASH_WaitForLastOperation(ProgramTimeout);
  
  if(status == FLASH_COMPLETE)
  {
    /* ǰһɣµ */
    FLASH->CR |= CR_PG_Set;
  
    *(__IO uint16_t*)Address = Data;
    /* ȴһ */
    status = FLASH_WaitForLastOperation(ProgramTimeout);
    if(status != FLASH_TIMEOUT)
    {
      /* ̲ɣʧ PG λ */
      FLASH->CR &= CR_PG_Reset;
    }
  } 
  /* ر״̬ */
  return status;
}

/**
  * @  ָѡֽݵַϱдһ.
  * @  Address: ̵ĵַ.
  *                  0x1FFFF804  0x1FFFF806. 
  * @  Data: д.
  * @  FLASH Status: ֵ: FLASH_ERROR_PG, FLASH_ERROR_WRP, 
  *                      FLASH_COMPLETE  FLASH_TIMEOUT. 
  */
FLASH_Status FLASH_ProgramOptionByteData(uint32_t Address, uint8_t Data)
{
  FLASH_Status status = FLASH_COMPLETE;
  /*  */
  assert_param(IS_OB_DATA_ADDRESS(Address));
  status = FLASH_WaitForLastOperation(ProgramTimeout);
  if(status == FLASH_COMPLETE)
  {
    /* ȨСϢ */
    FLASH->OPTKEYR = FLASH_KEY1;
    FLASH->OPTKEYR = FLASH_KEY2;
    /* ʹֽڱ̲ѡ */
    FLASH->CR |= CR_OPTPG_Set; 
    *(__IO uint16_t*)Address = Data;
    
    /* ȴһ */
    status = FLASH_WaitForLastOperation(ProgramTimeout);
    if(status != FLASH_TIMEOUT)
    {
      /* ̲ɣʧ OPTPG λ */
      FLASH->CR &= CR_OPTPG_Reset;
    }
  }    
  /* ѡֽݵı״̬ */
  return status;
}

/**
  * @  ҳд
  * @  FLASH_Pages: дҳĵַ.
  *                 :
  *                 STM32_ܶȴ: value between FLASH_WRProt_Pages0to3  FLASH_WRProt_Pages28to31  
  *                 STM32_ܶȴ: value between FLASH_WRProt_Pages0to3  FLASH_WRProt_Pages124to127
  *                 STM32_ܶȴ: value between FLASH_WRProt_Pages0to1  FLASH_WRProt_Pages60to61  FLASH_WRProt_Pages62to255
  *                 STM32_ͨԴ: value between FLASH_WRProt_Pages0to1  FLASH_WRProt_Pages60to61  FLASH_WRProt_Pages62to127    
  *                 FLASH_WRProt_AllPages
  * @  FLASH Status: ֵ: FLASH_ERROR_PG, FLASH_ERROR_WRP, 
  *                      FLASH_COMPLETE  FLASH_TIMEOUT.
  */
FLASH_Status FLASH_EnableWriteProtection(uint32_t FLASH_Pages)
{
  uint16_t WRP0_Data = 0xFFFF, WRP1_Data = 0xFFFF, WRP2_Data = 0xFFFF, WRP3_Data = 0xFFFF;
  
  FLASH_Status status = FLASH_COMPLETE;
  
  /*  */
  assert_param(IS_FLASH_WRPROT_PAGE(FLASH_Pages));
  
  FLASH_Pages = (uint32_t)(~FLASH_Pages);
  WRP0_Data = (uint16_t)(FLASH_Pages & WRP0_Mask);
  WRP1_Data = (uint16_t)((FLASH_Pages & WRP1_Mask) >> 8);
  WRP2_Data = (uint16_t)((FLASH_Pages & WRP2_Mask) >> 16);
  WRP3_Data = (uint16_t)((FLASH_Pages & WRP3_Mask) >> 24);
  
  /* ȴһ */
  status = FLASH_WaitForLastOperation(ProgramTimeout);
  
  if(status == FLASH_COMPLETE)
  {
    /* ȨСϢ */
    FLASH->OPTKEYR = FLASH_KEY1;
    FLASH->OPTKEYR = FLASH_KEY2;
    FLASH->CR |= CR_OPTPG_Set;
    if(WRP0_Data != 0xFF)
    {
      OB->WRP0 = WRP0_Data;
      
      /* ȴһ */
      status = FLASH_WaitForLastOperation(ProgramTimeout);
    }
    if((status == FLASH_COMPLETE) && (WRP1_Data != 0xFF))
    {
      OB->WRP1 = WRP1_Data;
      
      /* ȴһ */
      status = FLASH_WaitForLastOperation(ProgramTimeout);
    }
    if((status == FLASH_COMPLETE) && (WRP2_Data != 0xFF))
    {
      OB->WRP2 = WRP2_Data;
      
      /* ȴһ */
      status = FLASH_WaitForLastOperation(ProgramTimeout);
    }
    
    if((status == FLASH_COMPLETE)&& (WRP3_Data != 0xFF))
    {
      OB->WRP3 = WRP3_Data;
     
      /* ȴһ */
      status = FLASH_WaitForLastOperation(ProgramTimeout);
    }
          
    if(status != FLASH_TIMEOUT)
    {
      /* ̲ɣʧ OPTPG λ */
      FLASH->CR &= CR_OPTPG_Reset;
    }
  } 
  /* д״̬ */
  return status;       
}

/**
  * @  ʹܻʧܶ.
  * @ע   ڵ֮ǰʹѾѡֽڣ
  *         еѡֽԺ±Щѡֽ.
  * @  Newstate: ״̬.
  *                  : ENABLE  DISABLE.
  * @  FLASH Status: ֵ: FLASH_ERROR_PG, FLASH_ERROR_WRP, 
  *                      FLASH_COMPLETE or FLASH_TIMEOUT.
  */
FLASH_Status FLASH_ReadOutProtection(FunctionalState NewState)
{
  FLASH_Status status = FLASH_COMPLETE;
  /*  */
  assert_param(IS_FUNCTIONAL_STATE(NewState));
  status = FLASH_WaitForLastOperation(EraseTimeout);
  if(status == FLASH_COMPLETE)
  {
    /* ȨСϢ */
    FLASH->OPTKEYR = FLASH_KEY1;
    FLASH->OPTKEYR = FLASH_KEY2;
    FLASH->CR |= CR_OPTER_Set;
    FLASH->CR |= CR_STRT_Set;
    /* ȴһ */
    status = FLASH_WaitForLastOperation(EraseTimeout);
    if(status == FLASH_COMPLETE)
    {
      /* ɣʧ OPTER λ */
      FLASH->CR &= CR_OPTER_Reset;
      /* ʹֽڱ̲ѡ */
      FLASH->CR |= CR_OPTPG_Set; 
      if(NewState != DISABLE)
      {
        OB->RDP = 0x00;
      }
      else
      {
        OB->RDP = RDP_Key;  
      }
      /* ȴһ */
      status = FLASH_WaitForLastOperation(EraseTimeout); 
    
      if(status != FLASH_TIMEOUT)
      {
        /* ̲ɣʧ OPTPG λ */
        FLASH->CR &= CR_OPTPG_Reset;
      }
    }
    else 
    {
      if(status != FLASH_TIMEOUT)
      {
        /* ʧ OPTER λ */
        FLASH->CR &= CR_OPTER_Reset;
      }
    }
  }
  /* д״̬ */
  return status;      
}

/**
  * @  дFLASHûѡֽ: IWDG_SW / RST_STOP / RST_STDBY.
  * @  OB_IWDG: ѡ IWDG ģʽ
  *          ֵ֮һ:
  *          OB_IWDG_SW: ѡŹ
  *          OB_IWDG_HW: ѡӲŹ
  * @  OB_STOP:  STOP ģʽλ¼.
  *          ֵ֮һ:
  *          OB_STOP_NoRST:  STOP ģʽλ
  *          OB_STOP_RST:    STOP ģʽλ
  * @  OB_STDBY:  Standby ģʽλ¼.
  *          ֵ֮һ:
  *          OB_STDBY_NoRST:  Standby ģʽλ
  *          OB_STDBY_RST:    Standby ģʽλ
  * @  FLASH Status: ֵ: FLASH_ERROR_PG, FLASH_ERROR_WRP, 
  *                      FLASH_COMPLETE or FLASH_TIMEOUT.
  */
FLASH_Status FLASH_UserOptionByteConfig(uint16_t OB_IWDG, uint16_t OB_STOP, uint16_t OB_STDBY)
{
  FLASH_Status status = FLASH_COMPLETE; 

  /*  */
  assert_param(IS_OB_IWDG_SOURCE(OB_IWDG));
  assert_param(IS_OB_STOP_SOURCE(OB_STOP));
  assert_param(IS_OB_STDBY_SOURCE(OB_STDBY));

  /* ȨСϢ */
  FLASH->OPTKEYR = FLASH_KEY1;
  FLASH->OPTKEYR = FLASH_KEY2;
  
  /* ȴһ */
  status = FLASH_WaitForLastOperation(ProgramTimeout);
  
  if(status == FLASH_COMPLETE)
  {  
    /* ʹֽڱ̲ѡ */
    FLASH->CR |= CR_OPTPG_Set; 
           
    OB->USER = OB_IWDG | (uint16_t)(OB_STOP | (uint16_t)(OB_STDBY | ((uint16_t)0xF8))); 
  
    /* ȴһ */
    status = FLASH_WaitForLastOperation(ProgramTimeout);
    if(status != FLASH_TIMEOUT)
    {
      /* ̲ɣʧ OPTPG λ */
      FLASH->CR &= CR_OPTPG_Reset;
    }
  }    
  /* ѡֽڱ״̬ */
  return status;
}

/**
  * @   FLASH ûѡֽڵֵ.
  * @  û
  * @  FLASH ûѡֽڵֵ:IWDG_SW(Bit0), RST_STOP(Bit1)  RST_STDBY(Bit2).
  */
uint32_t FLASH_GetUserOptionByte(void)
{
  /* ûѡֵֽ */
  return (uint32_t)(FLASH->OBR >> 2);
}

/**
  * @   FLASH дѡֽڼĴֵ.
  * @  û
  * @  FLASH дѡֽڼĴֵ
  */
uint32_t FLASH_GetWriteProtectionOptionByte(void)
{
  /*  FLASH дĴֵ */
  return (uint32_t)(FLASH->WRPR);
}

/**
  * @   FLASH .
  * @  û
  * @  FLASH ״̬ (SET  RESET)
  */
FlagStatus FLASH_GetReadOutProtectionStatus(void)
{
  FlagStatus readoutstatus = RESET;
  if ((FLASH->OBR & RDPRT_Mask) != (uint32_t)RESET)
  {
    readoutstatus = SET;
  }
  else
  {
    readoutstatus = RESET;
  }
  return readoutstatus;
}

/**
  * @   FLASH Ԥȡָ.
  * @  û
  * @  FLASH Ԥȡ״̬ (SET  RESET).
  */
FlagStatus FLASH_GetPrefetchBufferStatus(void)
{
  FlagStatus bitstatus = RESET;
  
  if ((FLASH->ACR & ACR_PRFTBS_Mask) != (uint32_t)RESET)
  {
    bitstatus = SET;
  }
  else
  {
    bitstatus = RESET;
  }
  /*  FLASH Ԥȡָ (SET  RESET) */
  return bitstatus; 
}

/**
  * @  ʹܻʧָ FLASH ж.
  * @  FLASH_IT: ʹܻʧָܵ FLASH жԴ.
  *          ֵ:
  *          FLASH_IT_ERROR: FLASH жԴ
  *          FLASH_IT_EOP:   FLASH жԴ
  * @  NewState: ָ FLASH жϵ״̬.
  *                  : ENABLE  DISABLE.      
  * @  û 
  */
void FLASH_ITConfig(uint16_t FLASH_IT, FunctionalState NewState)
{
  /*  */
  assert_param(IS_FLASH_IT(FLASH_IT)); 
  assert_param(IS_FUNCTIONAL_STATE(NewState));
  if(NewState != DISABLE)
  {
    /* ʹжԴ */
    FLASH->CR |= FLASH_IT;
  }
  else
  {
    /* ʧжԴ */
    FLASH->CR &= ~(uint32_t)FLASH_IT;
  }
}

/**
  * @  ָ FLASH ־λ.
  * @  FLASH_FLAG: Ҫı־.
  *          ֵ֮һ:
  *          FLASH_FLAG_BSY:      FLASH æ־           
  *          FLASH_FLAG_PGERR:    FLASH ־       
  *          FLASH_FLAG_WRPRTERR: FLASH ҳд־    
  *          FLASH_FLAG_EOP:      FLASH ־          
  *          FLASH_FLAG_OPTERR:   FLASH ѡֽڴ־   
  * @  FLASH_FLAG ״̬ (SET  RESET).
  */
FlagStatus FLASH_GetFlagStatus(uint16_t FLASH_FLAG)
{
  FlagStatus bitstatus = RESET;
  /*  */
  assert_param(IS_FLASH_GET_FLAG(FLASH_FLAG)) ;
  if(FLASH_FLAG == FLASH_FLAG_OPTERR) 
  {
    if((FLASH->OBR & FLASH_FLAG_OPTERR) != (uint32_t)RESET)
    {
      bitstatus = SET;
    }
    else
    {
      bitstatus = RESET;
    }
  }
  else
  {
   if((FLASH->SR & FLASH_FLAG) != (uint32_t)RESET)
    {
      bitstatus = SET;
    }
    else
    {
      bitstatus = RESET;
    }
  }
  /*  FLASH_FLAG ״̬ (SET or RESET) */
  return bitstatus;
}

/**
  * @   FLASH ־λ.
  * @  FLASH_FLAG: Ҫı־.
  *          ֵ:        
  *          FLASH_FLAG_PGERR:    FLASH ־       
  *          FLASH_FLAG_WRPRTERR: FLASH Wҳд־      
  *          FLASH_FLAG_EOP:      FLASH ı־          
  * @  û
  */
void FLASH_ClearFlag(uint16_t FLASH_FLAG)
{
  /*  */
  assert_param(IS_FLASH_CLEAR_FLAG(FLASH_FLAG)) ;
  
  /* ־ */
  FLASH->SR = FLASH_FLAG;
}

/**
  * @   FLASH ״̬.
  * @  û
  * @  FLASH ״̬: ֵ: FLASH_BUSY, FLASH_ERROR_PG,
  *                    FLASH_ERROR_WRP  FLASH_COMPLETE
  */
FLASH_Status FLASH_GetStatus(void)
{
  FLASH_Status flashstatus = FLASH_COMPLETE;
  
  if((FLASH->SR & FLASH_FLAG_BSY) == FLASH_FLAG_BSY) 
  {
    flashstatus = FLASH_BUSY;
  }
  else 
  {  
    if((FLASH->SR & FLASH_FLAG_PGERR) != 0)
    { 
      flashstatus = FLASH_ERROR_PG;
    }
    else 
    {
      if((FLASH->SR & FLASH_FLAG_WRPRTERR) != 0 )
      {
        flashstatus = FLASH_ERROR_WRP;
      }
      else
      {
        flashstatus = FLASH_COMPLETE;
      }
    }
  }
  /*  FLASH ״̬ */
  return flashstatus;
}

/**
  * @  ȴһ Flash һ TIMEOUT ¼.
  * @  Timeout: FLASH ̳ʱʱ
  * @  FLASH Status: ֵ: FLASH_ERROR_PG, FLASH_ERROR_WRP, 
  *                      FLASH_COMPLETE  FLASH_TIMEOUT.
  */
FLASH_Status FLASH_WaitForLastOperation(uint32_t Timeout)
{ 
  FLASH_Status status = FLASH_COMPLETE;
   
  /*  FLASH ״̬ */
  status = FLASH_GetStatus();
  /* ȴ FLASH ɻ߷ʱ */
  while((status == FLASH_BUSY) && (Timeout != 0x00))
  {
    delay();
    status = FLASH_GetStatus();
    Timeout--;
  }
  if(Timeout == 0x00 )
  {
    status = FLASH_TIMEOUT;
  }
  /* ز״̬ */
  return status;
}

/**
  * @  һʱ.
  * @  û
  * @  û
  */
static void delay(void)
{
  __IO uint32_t i = 0;
  for(i = 0xFF; i != 0; i--)
  {
  }
}

/**
  * @}
  */

/**
  * @}
  */

/**
  * @}
  */

/******************* (C) COPYRIGHT 2009 STMicroelectronics *****END OF FILE****/
